@@ -32,7 +32,7 @@
 ///     2. Create an OpenGL context.
 ///     3. Print information about the context.
 
-#define WAFFLE_API_VERSION 0x0103
+#define WAFFLE_API_VERSION 0x0106
 
 #include <assert.h>
 #include <ctype.h>
@@ -53,6 +53,8 @@ removeXcodeArgs(int *argc, char **argv);
 
 #include "waffle.h"
 
+#define ARRAY_SIZE(a) (sizeof(a) / sizeof(a[0]))
+
 static const char *usage_message =
     "Usage:\n"
     "    wflinfo <Required Parameters> [Options]\n"
@@ -61,17 +63,18 @@ static const char *usage_message =
     "    Create an OpenGL or OpenGL ES context and print information about it.\n"
     "\n"
     "Required Parameters:\n"
-    "    -p, --platform\n"
-    "        One of: android, cgl, gbm, glx, wayland, wgl or x11_egl\n"
+    "    -p, --platform <platform>\n"
+    "        One of: android, cgl, gbm, glx, surfaceless_egl (or short\n"
+    "        alias 'sl'), wayland, wgl, or x11_egl.\n"
     "\n"
-    "    -a, --api\n"
+    "    -a, --api <api>\n"
     "        One of: gl, gles1, gles2 or gles3\n"
     "\n"
     "Options:\n"
-    "    -V, --version\n"
+    "    -V, --version <version>\n"
     "        For example --api=gl --version=3.2 would request OpenGL 3.2.\n"
     "\n"
-    "    --profile\n"
+    "    --profile <profile>\n"
     "        One of: core, compat or none\n"
     "\n"
     "    -v, --verbose\n"
@@ -83,6 +86,9 @@ static const char *usage_message =
     "    --debug-context\n"
     "        Create a debug context.\n"
     "\n"
+    "    -f, --format <format>\n"
+    "        One of: original (default) or json.\n"
+    "\n"
     "    -h, --help\n"
     "        Print wflinfo usage information.\n"
     "\n"
@@ -102,6 +108,7 @@ enum {
     OPT_VERBOSE = 'v',
     OPT_DEBUG_CONTEXT,
     OPT_FORWARD_COMPATIBLE,
+    OPT_FORMAT = 'f',
     OPT_HELP = 'h',
 };
 
@@ -113,6 +120,7 @@ static const struct option get_opts[] = {
     { .name = "verbose",        .has_arg = no_argument,           .val = OPT_VERBOSE },
     { .name = "debug-context",  .has_arg = no_argument,           .val = OPT_DEBUG_CONTEXT },
     { .name = "forward-compatible", .has_arg = no_argument,       .val = OPT_FORWARD_COMPATIBLE },
+    { .name = "format",         .has_arg = required_argument,     .val = OPT_FORMAT },
     { .name = "help",           .has_arg = no_argument,           .val = OPT_HELP },
     { 0 },
 };
@@ -131,6 +139,13 @@ strneq(const char *a, const char *b, size_t n)
 #define NORETURN
 #endif
 
+static void NORETURN
+error_oom(void)
+{
+    fprintf(stderr, "out of memory\n");
+    exit(EXIT_FAILURE);
+}
+
 static void NORETURN
 error_printf(const char *module, const char *fmt, ...)
 {
@@ -250,6 +265,11 @@ struct options {
 
     bool verbose;
 
+    enum format {
+        FORMAT_ORIGINAL,
+        FORMAT_JSON,
+    } format;
+
     bool context_forward_compatible;
     bool context_debug;
 
@@ -270,6 +290,8 @@ static const struct enum_map platform_map[] = {
     {WAFFLE_PLATFORM_WAYLAND,   "wayland"       },
     {WAFFLE_PLATFORM_WGL,       "wgl"           },
     {WAFFLE_PLATFORM_X11_EGL,   "x11_egl"       },
+    {WAFFLE_PLATFORM_SURFACELESS_EGL,   "surfaceless_egl" },
+    {WAFFLE_PLATFORM_SURFACELESS_EGL,   "sl"              },
     {0,                         0               },
 };
 
@@ -330,12 +352,13 @@ parse_args(int argc, char *argv[], struct options *opts)
     opts->context_profile = WAFFLE_NONE;
     opts->context_major = WAFFLE_DONT_CARE;
     opts->context_minor = WAFFLE_DONT_CARE;
+    opts->format = FORMAT_ORIGINAL;
 
     // prevent getopt_long from printing an error message
     opterr = 0;
 
     while (loop_get_opt) {
-        int opt = getopt_long(argc, argv, "a:hp:vV:", get_opts, NULL);
+        int opt = getopt_long(argc, argv, "a:f:hp:vV:", get_opts, NULL);
         switch (opt) {
             case -1:
                 loop_get_opt = false;
@@ -384,6 +407,15 @@ parse_args(int argc, char *argv[], struct options *opts)
                                        optarg);
                 }
                 break;
+            case OPT_FORMAT:
+                if (strcmp(optarg, "original") == 0) {
+                    opts->format = FORMAT_ORIGINAL;
+                } else if (strcmp(optarg, "json") == 0) {
+                    opts->format = FORMAT_JSON;
+                } else {
+                    usage_error_printf("'%s' is not a valid format", optarg);
+                }
+                break;
             case OPT_VERBOSE:
                 opts->verbose = true;
                 break;
@@ -458,6 +490,39 @@ parse_version(const char *version)
     return (major * 10) + minor;
 }
 
+static const char *
+get_vendor(void)
+{
+    const char *vendor = (const char *) glGetString(GL_VENDOR);
+    if (glGetError() != GL_NO_ERROR || vendor == NULL) {
+        vendor = "WFLINFO_GL_ERROR";
+    }
+
+    return vendor;
+}
+
+static const char *
+get_renderer(void)
+{
+    const char *renderer = (const char *) glGetString(GL_RENDERER);
+    if (glGetError() != GL_NO_ERROR || renderer == NULL) {
+        renderer = "WFLINFO_GL_ERROR";
+    }
+
+    return renderer;
+}
+
+static const char *
+get_version(void)
+{
+    const char *version_str = (const char *) glGetString(GL_VERSION);
+    if (glGetError() != GL_NO_ERROR || version_str == NULL) {
+        version_str = "WFLINFO_GL_ERROR";
+    }
+
+    return version_str;
+}
+
 static void
 print_extensions(bool use_stringi)
 {
@@ -487,23 +552,23 @@ print_extensions(bool use_stringi)
     printf("\n");
 }
 
+static struct {
+    GLint flag;
+    char *str;
+} context_flags[] = {
+    { GL_CONTEXT_FLAG_FORWARD_COMPATIBLE_BIT, "FORWARD_COMPATIBLE" },
+    { GL_CONTEXT_FLAG_DEBUG_BIT, "DEBUG" },
+    { GL_CONTEXT_FLAG_ROBUST_ACCESS_BIT_ARB, "ROBUST_ACCESS" },
+};
+
 static void
 print_context_flags(void)
 {
-    static struct {
-        GLint flag;
-        char *str;
-    } flags[] = {
-        { GL_CONTEXT_FLAG_FORWARD_COMPATIBLE_BIT, "FORWARD_COMPATIBLE" },
-        { GL_CONTEXT_FLAG_DEBUG_BIT, "DEBUG" },
-        { GL_CONTEXT_FLAG_ROBUST_ACCESS_BIT_ARB, "ROBUST_ACCESS" },
-    };
-    int flag_count = sizeof(flags) / sizeof(flags[0]);
-    GLint context_flags = 0;
+    GLint gl_context_flags = 0;
 
     printf("OpenGL context flags:");
 
-    glGetIntegerv(GL_CONTEXT_FLAGS, &context_flags);
+    glGetIntegerv(GL_CONTEXT_FLAGS, &gl_context_flags);
     if (glGetError() != GL_NO_ERROR) {
         printf(" WFLINFO_GL_ERROR\n");
         return;
@@ -514,42 +579,138 @@ print_context_flags(void)
         return;
     }
 
-    for (int i = 0; i < flag_count; i++) {
-        if ((flags[i].flag & context_flags) != 0) {
-            printf(" %s", flags[i].str);
-            context_flags = context_flags & ~flags[i].flag;
+    for (int i = 0; i < ARRAY_SIZE(context_flags); i++) {
+        if ((context_flags[i].flag & gl_context_flags) != 0) {
+            printf(" %s", context_flags[i].str);
+            gl_context_flags = gl_context_flags & ~context_flags[i].flag;
         }
     }
-    for (int i = 0; context_flags != 0; context_flags >>= 1, i++) {
-        if ((context_flags & 1) != 0) {
+    for (int i = 0; gl_context_flags != 0; gl_context_flags >>= 1, i++) {
+        if ((gl_context_flags & 1) != 0) {
             printf(" 0x%x", 1 << i);
         }
     }
     printf("\n");
 }
 
+static void
+json_print_extensions(bool use_stringi)
+{
+    // Print extensions in JSON format
+    printf("        \"extensions\": [\n");
+    if (use_stringi) {
+        GLint count = 0;
+        const char *ext;
+
+        glGetIntegerv(GL_NUM_EXTENSIONS, &count);
+        if (glGetError() != GL_NO_ERROR) {
+            printf("        \"WFLINFO_GL_ERROR\"");
+        } else {
+            for (int i = 0; i < count; i++) {
+                ext = (const char *) glGetStringi(GL_EXTENSIONS, i);
+                if (glGetError() != GL_NO_ERROR)
+                    ext = "WFLINFO_GL_ERROR";
+                printf("            \"%s\"%s\n", ext, (i + 1) < count ? "," : "");
+            }
+        }
+    } else {
+        const char *extensions = (const char *) glGetString(GL_EXTENSIONS);
+
+        if (glGetError() != GL_NO_ERROR || !extensions) {
+            printf("            \"WFLINFO_GL_ERROR\"");
+        } else {
+            // Copy the string because strtok() is destructive.
+            char *splitter = strdup(extensions);
+            if (!splitter)
+                error_oom();
+
+            splitter = strtok(splitter, " ");
+
+            // The JSON doesn't strictly need to be newline seperated, but it
+            // makes it much easier to read. Split the lines and add commas
+            // correctly.
+            while (splitter) {
+                printf("       \"%s\"", splitter);
+                splitter = strtok(NULL, " ");
+                if (splitter) {
+                    printf(",\n");
+                }
+            }
+        }
+
+        printf("\n");
+    }
+
+    printf("        ]\n");
+}
+
+/// @brief Print JSON formatted OpenGL (ES) information
+static bool
+print_json(const struct options *opts)
+{
+    while (glGetError() != GL_NO_ERROR) {
+        /* Clear all errors */
+    }
+
+    const char *vendor =  get_vendor();
+    const char *renderer = get_renderer();
+    const char *version_str = get_version();
+
+    const char *platform = enum_map_to_str(platform_map, opts->platform);
+    assert(platform != NULL);
+
+    const char *api = enum_map_to_str(context_api_map, opts->context_api);
+    assert(api != NULL);
+
+    // OpenGL and OpenGL ES >= 3.0 support glGetStringi(GL_EXTENSION, i).
+    const int version = parse_version(version_str);
+    const bool use_getstringi = version >= 30;
+
+    if (!glGetStringi && use_getstringi) {
+        error_get_gl_symbol("glGetStringi");
+    }
+
+    // See the equivalent section in print_wflinfo() for more info
+    const char *language_str = "None";
+    if ((opts->context_api == WAFFLE_CONTEXT_OPENGL && version >= 20)
+         || opts->context_api == WAFFLE_CONTEXT_OPENGL_ES2
+         || opts->context_api == WAFFLE_CONTEXT_OPENGL_ES3) {
+        language_str = (const char *) glGetString(GL_SHADING_LANGUAGE_VERSION);
+        if (glGetError() != GL_NO_ERROR || language_str == NULL) {
+            language_str = "WFLINFO_GL_ERROR";
+        }
+    }
+
+    printf("{\n");
+    printf("    \"waffle\": {\n");
+    printf("        \"platform\": \"%s\",\n", platform);
+    printf("        \"api\": \"%s\"\n", api);
+    printf("    },\n");
+    printf("    \"OpenGL\": {\n");
+    printf("        \"vendor string\": \"%s\",\n", vendor);
+    printf("        \"renderer string\": \"%s\",\n", renderer);
+    printf("        \"version string\": \"%s\",\n", version_str);
+    printf("        \"shading language version string\": \"%s\",\n", language_str);
+
+    json_print_extensions(use_getstringi);
+
+    printf("    }\n");
+    printf("}\n");
+
+    return true;
+}
+
 /// @brief Print out information about the context that was created.
 static bool
 print_wflinfo(const struct options *opts)
 {
-    while(glGetError() != GL_NO_ERROR) {
+    while (glGetError() != GL_NO_ERROR) {
         /* Clear all errors */
     }
 
-    const char *vendor = (const char *) glGetString(GL_VENDOR);
-    if (glGetError() != GL_NO_ERROR || vendor == NULL) {
-        vendor = "WFLINFO_GL_ERROR";
-    }
-
-    const char *renderer = (const char *) glGetString(GL_RENDERER);
-    if (glGetError() != GL_NO_ERROR || renderer == NULL) {
-        renderer = "WFLINFO_GL_ERROR";
-    }
-
-    const char *version_str = (const char *) glGetString(GL_VERSION);
-    if (glGetError() != GL_NO_ERROR || version_str == NULL) {
-        version_str = "WFLINFO_GL_ERROR";
-    }
+    const char *vendor =  get_vendor();
+    const char *renderer = get_renderer();
+    const char *version_str = get_version();
 
     const char *platform = enum_map_to_str(platform_map, opts->platform);
     assert(platform != NULL);
@@ -946,8 +1107,6 @@ success:
     return true;
 }
 
-#define ARRAY_SIZE(a) (sizeof(a) / sizeof(a[0]))
-
 /// Exit on failure.
 static void
 wflinfo_create_context(struct waffle_display *dpy,
@@ -1119,7 +1278,15 @@ main(int argc, char **argv)
         glGetStringi = waffle_get_proc_address("glGetStringi");
     }
 
-    ok = print_wflinfo(&opts);
+    switch (opts.format) {
+        case FORMAT_ORIGINAL:
+            ok = print_wflinfo(&opts);
+            break;
+        case FORMAT_JSON:
+            ok = print_json(&opts);
+            break;
+    }
+
     if (!ok)
         error_waffle();
 
@@ -1143,6 +1310,10 @@ main(int argc, char **argv)
     if (!ok)
         error_waffle();
 
+    ok = waffle_teardown();
+    if (!ok)
+        error_waffle();
+
     #ifdef __APPLE__
         cocoa_finish();
     #endif
