@@ -28,6 +28,28 @@
 #include "wcore_error.h"
 #include "wcore_util.h"
 
+bool
+wcore_add_size(size_t *res, size_t x, size_t y)
+{
+    if (x > SIZE_MAX - y) {
+        return false;
+    }
+
+    *res = x + y;
+    return true;
+}
+
+bool
+wcore_mul_size(size_t *res, size_t x, size_t y)
+{
+    if (x > SIZE_MAX / y) {
+        return false;
+    }
+
+    *res = x * y;
+    return true;
+}
+
 void*
 wcore_malloc(size_t size)
 {
@@ -46,6 +68,24 @@ wcore_calloc(size_t size)
     return p;
 }
 
+void*
+wcore_realloc(void *ptr, size_t size)
+{
+    void *p = realloc(ptr, size);
+    if (p == NULL)
+        wcore_error(WAFFLE_ERROR_BAD_ALLOC);
+    return p;
+}
+
+char*
+wcore_strdup(const char *str)
+{
+    char *p = strdup(str);
+    if (p == NULL)
+        wcore_error(WAFFLE_ERROR_BAD_ALLOC);
+    return p;
+}
+
 const char*
 wcore_enum_to_string(int32_t e)
 {
@@ -62,6 +102,8 @@ wcore_enum_to_string(int32_t e)
         CASE(WAFFLE_PLATFORM_X11_EGL);
         CASE(WAFFLE_PLATFORM_GBM);
         CASE(WAFFLE_PLATFORM_WGL);
+        CASE(WAFFLE_PLATFORM_NACL);
+        CASE(WAFFLE_PLATFORM_SURFACELESS_EGL);
         CASE(WAFFLE_CONTEXT_API);
         CASE(WAFFLE_CONTEXT_OPENGL);
         CASE(WAFFLE_CONTEXT_OPENGL_ES1);
@@ -74,6 +116,7 @@ wcore_enum_to_string(int32_t e)
         CASE(WAFFLE_CONTEXT_COMPATIBILITY_PROFILE);
         CASE(WAFFLE_CONTEXT_FORWARD_COMPATIBLE);
         CASE(WAFFLE_CONTEXT_DEBUG);
+        CASE(WAFFLE_CONTEXT_ROBUST_ACCESS);
         CASE(WAFFLE_RED_SIZE);
         CASE(WAFFLE_GREEN_SIZE);
         CASE(WAFFLE_BLUE_SIZE);
@@ -88,6 +131,9 @@ wcore_enum_to_string(int32_t e)
         CASE(WAFFLE_DL_OPENGL_ES1);
         CASE(WAFFLE_DL_OPENGL_ES2);
         CASE(WAFFLE_DL_OPENGL_ES3);
+        CASE(WAFFLE_WINDOW_WIDTH);
+        CASE(WAFFLE_WINDOW_HEIGHT);
+        CASE(WAFFLE_WINDOW_FULLSCREEN);
 
         default: return NULL;
 
