@@ -32,6 +32,7 @@
 #include "wcore_error.h"
 
 #include "glx_config.h"
+#include "glx_context.h"
 #include "glx_display.h"
 #include "glx_platform.h"
 #include "glx_wrappers.h"
@@ -54,8 +55,6 @@ static bool
 glx_config_check_context_attrs(struct glx_display *dpy,
                                const struct wcore_config_attrs *attrs)
 {
-    struct glx_platform *plat = glx_platform(dpy->wcore.platform);
-
     if (attrs->context_forward_compatible) {
         assert(attrs->context_api == WAFFLE_CONTEXT_OPENGL);
         assert(wcore_config_attrs_version_ge(attrs, 30));
@@ -68,13 +67,20 @@ glx_config_check_context_attrs(struct glx_display *dpy,
         return false;
     }
 
+    if (attrs->context_robust && !dpy->ARB_create_context_robustness) {
+        wcore_errorf(WAFFLE_ERROR_UNSUPPORTED_ON_PLATFORM,
+                     "GLX_ARB_create_context_robustness is required in order to "
+                     "request a robust access context");
+        return false;
+    }
+
     switch (attrs->context_api) {
         case WAFFLE_CONTEXT_OPENGL:
-            if (!wcore_config_attrs_version_eq(attrs, 10) && !dpy->ARB_create_context) {
+            if (glx_context_needs_arb_create_context(attrs) &&
+                !dpy->ARB_create_context) {
                 wcore_errorf(WAFFLE_ERROR_UNSUPPORTED_ON_PLATFORM,
                              "GLX_ARB_create_context is required in order to "
-                             "request an OpenGL version not equal to the default "
-                             "value 1.0");
+                             "request an OpenGL version greater or equal than 3.2");
                 return false;
             }
             else if (wcore_config_attrs_version_ge(attrs, 32) && !dpy->ARB_create_context_profile) {
@@ -98,29 +104,16 @@ glx_config_check_context_attrs(struct glx_display *dpy,
             return true;
 
         case WAFFLE_CONTEXT_OPENGL_ES1:
-            assert(wcore_config_attrs_version_eq(attrs, 10) ||
-                   wcore_config_attrs_version_eq(attrs, 11));
-            assert(!attrs->context_forward_compatible);
-
             if (!dpy->EXT_create_context_es_profile) {
                 wcore_errorf(WAFFLE_ERROR_UNSUPPORTED_ON_PLATFORM,
                              "GLX_EXT_create_context_es_profile is required "
                              "to create an OpenGL ES1 context");
                 return false;
             }
-            else if (!linux_platform_dl_can_open(plat->linux,
-                                            WAFFLE_DL_OPENGL_ES1)) {
-                wcore_errorf(WAFFLE_ERROR_UNSUPPORTED_ON_PLATFORM,
-                             "failed to open the OpenGL ES1 library");
-                return false;
-            }
 
             return true;
 
         case WAFFLE_CONTEXT_OPENGL_ES2:
-            assert(attrs->context_major_version == 2);
-            assert(!attrs->context_forward_compatible);
-
             if (!dpy->EXT_create_context_es_profile
                 && !dpy->EXT_create_context_es2_profile) {
                 wcore_errorf(WAFFLE_ERROR_UNSUPPORTED_ON_PLATFORM,
@@ -129,17 +122,10 @@ glx_config_check_context_attrs(struct glx_display *dpy,
                              "to create an OpenGL ES2 context");
                 return false;
             }
-            if (!linux_platform_dl_can_open(plat->linux,
-                                            WAFFLE_DL_OPENGL_ES2)) {
-                wcore_errorf(WAFFLE_ERROR_UNSUPPORTED_ON_PLATFORM,
-                             "failed to open the OpenGL ES2 library");
-                return false;
-            }
+
             return true;
 
         case WAFFLE_CONTEXT_OPENGL_ES3:
-            assert(attrs->context_major_version == 3);
-
             if (!dpy->EXT_create_context_es_profile) {
                 wcore_errorf(WAFFLE_ERROR_UNSUPPORTED_ON_PLATFORM,
                              "GLX_EXT_create_context_es_profile is required "
@@ -147,18 +133,10 @@ glx_config_check_context_attrs(struct glx_display *dpy,
                 return false;
             }
 
-            if (!linux_platform_dl_can_open(plat->linux,
-                                            WAFFLE_DL_OPENGL_ES3)) {
-                wcore_errorf(WAFFLE_ERROR_UNSUPPORTED_ON_PLATFORM,
-                             "failed to open the OpenGL ES3 library");
-                return false;
-            }
-
             return true;
 
         default:
-            wcore_error_internal("context_api has bad value %#x",
-                                 attrs->context_api);
+            assert(false);
             return false;
     }
 }
