@@ -29,6 +29,14 @@
 // It is an alias of GLX_CONTEXT_ES2_PROFILE_BIT_EXT.
 #define GLX_CONTEXT_ES_PROFILE_BIT_EXT 0x00000004
 
+// The official headers provided by nvidia do not provide these via glxext.h
+#ifndef GLX_ARB_create_context_profile
+#define GLX_ARB_create_context_profile 1
+#define GLX_CONTEXT_CORE_PROFILE_BIT_ARB  0x00000001
+#define GLX_CONTEXT_COMPATIBILITY_PROFILE_BIT_ARB 0x00000002
+#define GLX_CONTEXT_PROFILE_MASK_ARB      0x9126
+#endif /* GLX_ARB_create_context_profile */
+
 #include <assert.h>
 #include <stdlib.h>
 
@@ -143,6 +151,10 @@ glx_context_fill_attrib_list(struct glx_config *config,
         context_flags |= GLX_CONTEXT_DEBUG_BIT_ARB;
     }
 
+    if (attrs->context_robust) {
+        context_flags |= GLX_CONTEXT_ROBUST_ACCESS_BIT_ARB;
+    }
+
     if (context_flags != 0) {
         attrib_list[i++] = GLX_CONTEXT_FLAGS_ARB;
         attrib_list[i++] = context_flags;
@@ -161,7 +173,17 @@ glx_context_create_native(struct glx_config *config,
     struct glx_display *dpy = glx_display(config->wcore.display);
     struct glx_platform *platform = glx_platform(dpy->wcore.platform);
 
-    if (dpy->ARB_create_context) {
+    // Use ARB_create_context when we have
+    // - OpenGL version 1.0, or
+    // - OpenGL version 3.2 or greater, or
+    // - OpenGL with fwd_compat, or
+    // - Debug context
+    //
+    // The first one of the four is optional, the remainder hard requirement
+    // for the use of ARB_create_context.
+    if (dpy->ARB_create_context &&
+        (wcore_config_attrs_version_eq(&config->wcore.attrs, 10) ||
+         glx_context_needs_arb_create_context(&config->wcore.attrs))) {
         bool ok;
 
         // Choose a large size to prevent accidental overflow.
